# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: DeviceManagement.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import APIRequest_pb2 as APIRequest__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x44\x65viceManagement.proto\x12\x10\x44\x65viceManagement\x1a\x10\x41PIRequest.proto\"\x97\x03\n\x06\x44\x65vice\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x01 \x01(\x0c\x12\x12\n\ndeviceName\x18\x02 \x01(\t\x12\x16\n\x0e\x64\x65vicePlatform\x18\x03 \x01(\t\x12\x32\n\x0c\x64\x65viceStatus\x18\x04 \x01(\x0e\x32\x1c.Authentication.DeviceStatus\x12.\n\nloginState\x18\x05 \x01(\x0e\x32\x1a.Authentication.LoginState\x12\x15\n\rclientVersion\x18\x06 \x01(\t\x12\x30\n\nclientType\x18\x07 \x01(\x0e\x32\x1c.DeviceManagement.ClientType\x12@\n\x12\x63lientTypeCategory\x18\x08 \x01(\x0e\x32$.DeviceManagement.ClientTypeCategory\x12:\n\x10\x63lientFormFactor\x18\t \x01(\x0e\x32 .Authentication.ClientFormFactor\x12\x18\n\x10lastModifiedTime\x18\n \x01(\x03\"8\n\x0b\x44\x65viceGroup\x12)\n\x07\x64\x65vices\x18\x01 \x03(\x0b\x32\x18.DeviceManagement.Device\"I\n\x12\x44\x65viceUserResponse\x12\x33\n\x0c\x64\x65viceGroups\x18\x01 \x03(\x0b\x32\x1d.DeviceManagement.DeviceGroup\"K\n\x13\x44\x65viceActionRequest\x12\x34\n\x0c\x64\x65viceAction\x18\x01 \x03(\x0b\x32\x1e.DeviceManagement.DeviceAction\"j\n\x0c\x44\x65viceAction\x12<\n\x10\x64\x65viceActionType\x18\x01 \x01(\x0e\x32\".DeviceManagement.DeviceActionType\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x02 \x03(\x0c\"\x8d\x01\n\x14\x44\x65viceActionResponse\x12@\n\x12\x64\x65viceActionResult\x18\x01 \x03(\x0b\x32$.DeviceManagement.DeviceActionResult\x12\x33\n\x0c\x64\x65viceGroups\x18\x02 \x03(\x0b\x32\x1d.DeviceManagement.DeviceGroup\"\xb2\x01\n\x12\x44\x65viceActionResult\x12<\n\x10\x64\x65viceActionType\x18\x01 \x01(\x0e\x32\".DeviceManagement.DeviceActionType\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x02 \x03(\x0c\x12@\n\x12\x64\x65viceActionStatus\x18\x03 \x01(\x0e\x32$.DeviceManagement.DeviceActionStatus\"K\n\x13\x44\x65viceRenameRequest\x12\x34\n\x0c\x64\x65viceRename\x18\x01 \x03(\x0b\x32\x1e.DeviceManagement.DeviceRename\"C\n\x0c\x44\x65viceRename\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x01 \x01(\x0c\x12\x15\n\rdeviceNewName\x18\x02 \x01(\t\"\x8d\x01\n\x14\x44\x65viceRenameResponse\x12@\n\x12\x64\x65viceRenameResult\x18\x01 \x03(\x0b\x32$.DeviceManagement.DeviceRenameResult\x12\x33\n\x0c\x64\x65viceGroups\x18\x02 \x03(\x0b\x32\x1d.DeviceManagement.DeviceGroup\"\x8b\x01\n\x12\x44\x65viceRenameResult\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x01 \x01(\x0c\x12\x15\n\rdeviceNewName\x18\x02 \x01(\t\x12@\n\x12\x64\x65viceActionStatus\x18\x03 \x01(\x0e\x32$.DeviceManagement.DeviceActionStatus\"/\n\x12\x44\x65viceAdminRequest\x12\x19\n\x11\x65nterpriseUserIds\x18\x01 \x03(\x03\"O\n\x13\x44\x65viceAdminResponse\x12\x38\n\x0e\x64\x65viceUserList\x18\x01 \x03(\x0b\x32 .DeviceManagement.DeviceUserList\"_\n\x0e\x44\x65viceUserList\x12\x18\n\x10\x65nterpriseUserId\x18\x01 \x01(\x03\x12\x33\n\x0c\x64\x65viceGroups\x18\x02 \x03(\x0b\x32\x1d.DeviceManagement.DeviceGroup\"Z\n\x18\x44\x65viceAdminActionRequest\x12>\n\x11\x64\x65viceAdminAction\x18\x01 \x03(\x0b\x32#.DeviceManagement.DeviceAdminAction\"\x89\x01\n\x11\x44\x65viceAdminAction\x12<\n\x10\x64\x65viceActionType\x18\x01 \x01(\x0e\x32\".DeviceManagement.DeviceActionType\x12\x18\n\x10\x65nterpriseUserId\x18\x02 \x01(\x03\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x03 \x03(\x0c\"h\n\x19\x44\x65viceAdminActionResponse\x12K\n\x18\x64\x65viceAdminActionResults\x18\x01 \x03(\x0b\x32).DeviceManagement.DeviceAdminActionResult\"\xd1\x01\n\x17\x44\x65viceAdminActionResult\x12<\n\x10\x64\x65viceActionType\x18\x01 \x01(\x0e\x32\".DeviceManagement.DeviceActionType\x12\x18\n\x10\x65nterpriseUserId\x18\x02 \x01(\x03\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x03 \x03(\x0c\x12@\n\x12\x64\x65viceActionStatus\x18\x04 \x01(\x0e\x32$.DeviceManagement.DeviceActionStatus*\xb2\x01\n\x10\x44\x65viceActionType\x12\x0e\n\nDA_INVALID\x10\x00\x12\r\n\tDA_LOGOUT\x10\x01\x12\r\n\tDA_REMOVE\x10\x02\x12\x0b\n\x07\x44\x41_LOCK\x10\x03\x12\r\n\tDA_UNLOCK\x10\x04\x12\x1a\n\x16\x44\x41_DEVICE_ACCOUNT_LOCK\x10\x05\x12\x1c\n\x18\x44\x41_DEVICE_ACCOUNT_UNLOCK\x10\x06\x12\x0b\n\x07\x44\x41_LINK\x10\x07\x12\r\n\tDA_UNLINK\x10\x08*?\n\x12\x44\x65viceActionStatus\x12\x0b\n\x07INVALID\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\x0f\n\x0bNOT_ALLOWED\x10\x02*\xec\x05\n\nClientType\x12\x08\n\x04NONE\x10\x00\x12\x0b\n\x07\x41NDROID\x10\x01\x12\x0e\n\nBLACKBERRY\x10\x02\x12\x10\n\x0c\x42LACKBERRY10\x10\x03\x12\x0b\n\x07\x44\x45SKTOP\x10\x04\x12\x07\n\x03IOS\x10\x05\x12\x0b\n\x07MAC_APP\x10\x06\x12\x0b\n\x07WEB_APP\x10\x07\x12\x11\n\rWINDOWS_PHONE\x10\x08\x12\x0b\n\x07SURFACE\x10\t\x12\x08\n\x04WIN8\x10\n\x12\x10\n\x0cIE_EXTENSION\x10\x0b\x12\x14\n\x10\x43HROME_EXTENSION\x10\x0c\x12\x15\n\x11\x46IREFOX_EXTENSION\x10\r\x12\x14\n\x10SAFARI_EXTENSION\x10\x0e\x12\n\n\x06\x44OCOMO\x10\x0f\x12\x0b\n\x07UNKNOWN\x10\x10\x12\n\n\x06SERVER\x10\x11\x12\r\n\tCOMMANDER\x10\x12\x12\n\n\x06\x42RIDGE\x10\x13\x12!\n\x1d\x45NTERPRISE_MANAGEMENT_CONSOLE\x10\x14\x12\x12\n\x0e\x45\x44GE_EXTENSION\x10\x15\x12\x10\n\x0cSUPPORT_TOOL\x10\x16\x12\x0f\n\x0bSSO_CONNECT\x10\x17\x12\x14\n\x10\x44\x45SKTOP_ELECTRON\x10\x18\x12\x15\n\x11PASSWORD_IMPORTER\x10\x19\x12\x08\n\x04\x43HAT\x10\x1a\x12\x14\n\x10MAC_APP_ELECTRON\x10\x1b\x12\x0c\n\x08\x43HAT_IOS\x10\x1c\x12\x10\n\x0c\x43HAT_ANDROID\x10\x1d\x12\x10\n\x0c\x43HAT_WINDOWS\x10\x1e\x12\x0c\n\x08\x43HAT_MAC\x10\x1f\x12\x08\n\x04SCIM\x10 \x12\n\n\x06LAMBDA\x10!\x12\x1e\n\x1a\x43ONNECTWISE_CONTROL_HELPER\x10\"\x12\x1a\n\x16\x45NTERPRISE_CLIENT_TOOL\x10#\x12\x16\n\x12SECRETS_MANAGER_JS\x10$\x12\x1a\n\x16SECRETS_MANAGER_PYTHON\x10%\x12\x16\n\x12SECRETS_MANAGER_GO\x10&\x12\x18\n\x14SECRETS_MANAGER_JAVA\x10\'\x12\x17\n\x13SECRETS_MANAGER_NET\x10(*\xcb\x01\n\x12\x43lientTypeCategory\x12\x0c\n\x08\x43\x41T_NONE\x10\x00\x12\r\n\tCAT_ADMIN\x10\x01\x12\x0f\n\x0b\x43\x41T_DESKTOP\x10\x02\x12\x11\n\rCAT_EXTENSION\x10\x03\x12\x0e\n\nCAT_MOBILE\x10\x04\x12\r\n\tCAT_OTHER\x10\x05\x12\x11\n\rCAT_WEB_VAULT\x10\x06\x12\x14\n\x10\x43\x41T_CHAT_DESKTOP\x10\x07\x12\x13\n\x0f\x43\x41T_CHAT_MOBILE\x10\x08\x12\x17\n\x13\x43\x41T_SECRETS_MANAGER\x10\tB,\n\x18\x63om.keepersecurity.protoB\x10\x44\x65viceManagementb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'DeviceManagement_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030com.keepersecurity.protoB\020DeviceManagement'
  _globals['_DEVICEACTIONTYPE']._serialized_start=2325
  _globals['_DEVICEACTIONTYPE']._serialized_end=2503
  _globals['_DEVICEACTIONSTATUS']._serialized_start=2505
  _globals['_DEVICEACTIONSTATUS']._serialized_end=2568
  _globals['_CLIENTTYPE']._serialized_start=2571
  _globals['_CLIENTTYPE']._serialized_end=3319
  _globals['_CLIENTTYPECATEGORY']._serialized_start=3322
  _globals['_CLIENTTYPECATEGORY']._serialized_end=3525
  _globals['_DEVICE']._serialized_start=63
  _globals['_DEVICE']._serialized_end=470
  _globals['_DEVICEGROUP']._serialized_start=472
  _globals['_DEVICEGROUP']._serialized_end=528
  _globals['_DEVICEUSERRESPONSE']._serialized_start=530
  _globals['_DEVICEUSERRESPONSE']._serialized_end=603
  _globals['_DEVICEACTIONREQUEST']._serialized_start=605
  _globals['_DEVICEACTIONREQUEST']._serialized_end=680
  _globals['_DEVICEACTION']._serialized_start=682
  _globals['_DEVICEACTION']._serialized_end=788
  _globals['_DEVICEACTIONRESPONSE']._serialized_start=791
  _globals['_DEVICEACTIONRESPONSE']._serialized_end=932
  _globals['_DEVICEACTIONRESULT']._serialized_start=935
  _globals['_DEVICEACTIONRESULT']._serialized_end=1113
  _globals['_DEVICERENAMEREQUEST']._serialized_start=1115
  _globals['_DEVICERENAMEREQUEST']._serialized_end=1190
  _globals['_DEVICERENAME']._serialized_start=1192
  _globals['_DEVICERENAME']._serialized_end=1259
  _globals['_DEVICERENAMERESPONSE']._serialized_start=1262
  _globals['_DEVICERENAMERESPONSE']._serialized_end=1403
  _globals['_DEVICERENAMERESULT']._serialized_start=1406
  _globals['_DEVICERENAMERESULT']._serialized_end=1545
  _globals['_DEVICEADMINREQUEST']._serialized_start=1547
  _globals['_DEVICEADMINREQUEST']._serialized_end=1594
  _globals['_DEVICEADMINRESPONSE']._serialized_start=1596
  _globals['_DEVICEADMINRESPONSE']._serialized_end=1675
  _globals['_DEVICEUSERLIST']._serialized_start=1677
  _globals['_DEVICEUSERLIST']._serialized_end=1772
  _globals['_DEVICEADMINACTIONREQUEST']._serialized_start=1774
  _globals['_DEVICEADMINACTIONREQUEST']._serialized_end=1864
  _globals['_DEVICEADMINACTION']._serialized_start=1867
  _globals['_DEVICEADMINACTION']._serialized_end=2004
  _globals['_DEVICEADMINACTIONRESPONSE']._serialized_start=2006
  _globals['_DEVICEADMINACTIONRESPONSE']._serialized_end=2110
  _globals['_DEVICEADMINACTIONRESULT']._serialized_start=2113
  _globals['_DEVICEADMINACTIONRESULT']._serialized_end=2322
# @@protoc_insertion_point(module_scope)