# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: BI.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x08\x42I.proto\x12\x02\x42I\x1a\x1cgoogle/protobuf/struct.proto\"f\n\x1bValidateSessionTokenRequest\x12\x1d\n\x15\x65ncryptedSessionToken\x18\x01 \x01(\x0c\x12\x1c\n\x14returnMcEnterpiseIds\x18\x02 \x01(\x08\x12\n\n\x02ip\x18\x03 \x01(\t\"\xda\x02\n\x1cValidateSessionTokenResponse\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x0e\n\x06userId\x18\x02 \x01(\x05\x12\x18\n\x10\x65nterpriseUserId\x18\x03 \x01(\x03\x12\x37\n\x06status\x18\x04 \x01(\x0e\x32\'.BI.ValidateSessionTokenResponse.Status\x12\x15\n\rstatusMessage\x18\x05 \x01(\t\x12\x17\n\x0fmcEnterpriseIds\x18\x06 \x03(\x05\x12\x18\n\x10hasMSPPermission\x18\x07 \x01(\x08\x12\x1e\n\x16\x64\x65letedMcEnterpriseIds\x18\x08 \x03(\x05\"[\n\x06Status\x12\t\n\x05VALID\x10\x00\x12\r\n\tNOT_VALID\x10\x01\x12\x0b\n\x07\x45XPIRED\x10\x02\x12\x0e\n\nIP_BLOCKED\x10\x03\x12\x1a\n\x16INVALID_CLIENT_VERSION\x10\x04\"\x1b\n\x19SubscriptionStatusRequest\"\xa6\x03\n\x1aSubscriptionStatusResponse\x12$\n\x0b\x61utoRenewal\x18\x01 \x01(\x0b\x32\x0f.BI.AutoRenewal\x12/\n\x14\x63urrentPaymentMethod\x18\x02 \x01(\x0b\x32\x11.BI.PaymentMethod\x12\x14\n\x0c\x63heckoutLink\x18\x03 \x01(\t\x12\x19\n\x11licenseCreateDate\x18\x04 \x01(\x03\x12\x15\n\risDistributor\x18\x05 \x01(\x08\x12\x13\n\x0bisLegacyMsp\x18\x06 \x01(\x08\x12&\n\x0clicenseStats\x18\x08 \x03(\x0b\x32\x10.BI.LicenseStats\x12\x35\n\x0egradientStatus\x18\t \x01(\x0e\x32\x1d.BI.GradientIntegrationStatus\x12\x17\n\x0fhideTrialBanner\x18\n \x01(\x08\x12\x1c\n\x14gradientLastSyncDate\x18\x0b \x01(\t\x12\x1c\n\x14gradientNextSyncDate\x18\x0c \x01(\t\x12 \n\x18isGradientMappingPending\x18\r \x01(\x08\"\xd7\x01\n\x0cLicenseStats\x12#\n\x04type\x18\x01 \x01(\x0e\x32\x15.BI.LicenseStats.Type\x12\x11\n\tavailable\x18\x02 \x01(\x05\x12\x0c\n\x04used\x18\x03 \x01(\x05\"\x80\x01\n\x04Type\x12\x18\n\x14LICENSE_STAT_UNKNOWN\x10\x00\x12\x0c\n\x08MSP_BASE\x10\x01\x12\x0f\n\x0bMC_BUSINESS\x10\x02\x12\x14\n\x10MC_BUSINESS_PLUS\x10\x03\x12\x11\n\rMC_ENTERPRISE\x10\x04\x12\x16\n\x12MC_ENTERPRISE_PLUS\x10\x05\"@\n\x0b\x41utoRenewal\x12\x0e\n\x06nextOn\x18\x01 \x01(\x03\x12\x10\n\x08\x64\x61ysLeft\x18\x02 \x01(\x05\x12\x0f\n\x07isTrial\x18\x03 \x01(\x08\"\x84\x04\n\rPaymentMethod\x12$\n\x04type\x18\x01 \x01(\x0e\x32\x16.BI.PaymentMethod.Type\x12$\n\x04\x63\x61rd\x18\x02 \x01(\x0b\x32\x16.BI.PaymentMethod.Card\x12$\n\x04sepa\x18\x03 \x01(\x0b\x32\x16.BI.PaymentMethod.Sepa\x12(\n\x06paypal\x18\x04 \x01(\x0b\x32\x18.BI.PaymentMethod.Paypal\x12\x15\n\rfailedBilling\x18\x05 \x01(\x08\x12(\n\x06vendor\x18\x06 \x01(\x0b\x32\x18.BI.PaymentMethod.Vendor\x12\x36\n\rpurchaseOrder\x18\x07 \x01(\x0b\x32\x1f.BI.PaymentMethod.PurchaseOrder\x1a$\n\x04\x43\x61rd\x12\r\n\x05last4\x18\x01 \x01(\t\x12\r\n\x05\x62rand\x18\x02 \x01(\t\x1a&\n\x04Sepa\x12\r\n\x05last4\x18\x01 \x01(\t\x12\x0f\n\x07\x63ountry\x18\x02 \x01(\t\x1a\x08\n\x06Paypal\x1a\x16\n\x06Vendor\x12\x0c\n\x04name\x18\x01 \x01(\t\x1a\x1d\n\rPurchaseOrder\x12\x0c\n\x04name\x18\x01 \x01(\t\"O\n\x04Type\x12\x08\n\x04\x43\x41RD\x10\x00\x12\x08\n\x04SEPA\x10\x01\x12\n\n\x06PAYPAL\x10\x02\x12\x08\n\x04NONE\x10\x03\x12\n\n\x06VENDOR\x10\x04\x12\x11\n\rPURCHASEORDER\x10\x05\"\x1f\n\x1dSubscriptionMspPricingRequest\"\\\n\x1eSubscriptionMspPricingResponse\x12\x19\n\x06\x61\x64\x64ons\x18\x02 \x03(\x0b\x32\t.BI.Addon\x12\x1f\n\tfilePlans\x18\x03 \x03(\x0b\x32\x0c.BI.FilePlan\"\x1e\n\x1cSubscriptionMcPricingRequest\"|\n\x1dSubscriptionMcPricingResponse\x12\x1f\n\tbasePlans\x18\x01 \x03(\x0b\x32\x0c.BI.BasePlan\x12\x19\n\x06\x61\x64\x64ons\x18\x02 \x03(\x0b\x32\t.BI.Addon\x12\x1f\n\tfilePlans\x18\x03 \x03(\x0b\x32\x0c.BI.FilePlan\".\n\x08\x42\x61sePlan\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x16\n\x04\x63ost\x18\x02 \x01(\x0b\x32\x08.BI.Cost\"C\n\x05\x41\x64\x64on\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x16\n\x04\x63ost\x18\x02 \x01(\x0b\x32\x08.BI.Cost\x12\x16\n\x0e\x61mountConsumed\x18\x03 \x01(\x03\".\n\x08\x46ilePlan\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x16\n\x04\x63ost\x18\x02 \x01(\x0b\x32\x08.BI.Cost\"\xbf\x01\n\x04\x43ost\x12\x0e\n\x06\x61mount\x18\x03 \x01(\x01\x12%\n\tamountPer\x18\x04 \x01(\x0e\x32\x12.BI.Cost.AmountPer\x12\x1e\n\x08\x63urrency\x18\x05 \x01(\x0e\x32\x0c.BI.Currency\"`\n\tAmountPer\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05MONTH\x10\x01\x12\x0e\n\nUSER_MONTH\x10\x02\x12\x17\n\x13USER_CONSUMED_MONTH\x10\x03\x12\x12\n\x0e\x45NDPOINT_MONTH\x10\x04\"\\\n\x14InvoiceSearchRequest\x12\x0c\n\x04size\x18\x01 \x01(\x05\x12\x17\n\x0fstartingAfterId\x18\x02 \x01(\x05\x12\x1d\n\x15\x61llInvoicesUnfiltered\x18\x03 \x01(\x08\"6\n\x15InvoiceSearchResponse\x12\x1d\n\x08invoices\x18\x01 \x03(\x0b\x32\x0b.BI.Invoice\"\xbe\x02\n\x07Invoice\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x15\n\rinvoiceNumber\x18\x02 \x01(\t\x12\x13\n\x0binvoiceDate\x18\x03 \x01(\x03\x12\x14\n\x0clicenseCount\x18\x04 \x01(\x05\x12#\n\ttotalCost\x18\x05 \x01(\x0b\x32\x10.BI.Invoice.Cost\x12%\n\x0binvoiceType\x18\x06 \x01(\x0e\x32\x10.BI.Invoice.Type\x1a\x36\n\x04\x43ost\x12\x0e\n\x06\x61mount\x18\x01 \x01(\x01\x12\x1e\n\x08\x63urrency\x18\x02 \x01(\x0e\x32\x0c.BI.Currency\"a\n\x04Type\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03NEW\x10\x01\x12\x0b\n\x07RENEWAL\x10\x02\x12\x0b\n\x07UPGRADE\x10\x03\x12\x0b\n\x07RESTORE\x10\x04\x12\x0f\n\x0b\x41SSOCIATION\x10\x05\x12\x0b\n\x07OVERAGE\x10\x06\"\x1a\n\x18VaultInvoicesListRequest\"?\n\x19VaultInvoicesListResponse\x12\"\n\x08invoices\x18\x01 \x03(\x0b\x32\x10.BI.VaultInvoice\"\x8f\x01\n\x0cVaultInvoice\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x15\n\rinvoiceNumber\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x61teCreated\x18\x03 \x01(\x03\x12\x1f\n\x05total\x18\x04 \x01(\x0b\x32\x10.BI.Invoice.Cost\x12&\n\x0cpurchaseType\x18\x05 \x01(\x0e\x32\x10.BI.Invoice.Type\"/\n\x16InvoiceDownloadRequest\x12\x15\n\rinvoiceNumber\x18\x01 \x01(\t\"9\n\x17InvoiceDownloadResponse\x12\x0c\n\x04link\x18\x01 \x01(\t\x12\x10\n\x08\x66ileName\x18\x02 \x01(\t\"8\n\x1fVaultInvoiceDownloadLinkRequest\x12\x15\n\rinvoiceNumber\x18\x01 \x01(\t\"B\n VaultInvoiceDownloadLinkResponse\x12\x0c\n\x04link\x18\x01 \x01(\t\x12\x10\n\x08\x66ileName\x18\x02 \x01(\t\"<\n\x1dReportingDailySnapshotRequest\x12\r\n\x05month\x18\x01 \x01(\x05\x12\x0c\n\x04year\x18\x02 \x01(\x05\"v\n\x1eReportingDailySnapshotResponse\x12#\n\x07records\x18\x01 \x03(\x0b\x32\x12.BI.SnapshotRecord\x12/\n\rmcEnterprises\x18\x02 \x03(\x0b\x32\x18.BI.SnapshotMcEnterprise\"\xd7\x01\n\x0eSnapshotRecord\x12\x0c\n\x04\x64\x61te\x18\x01 \x01(\x03\x12\x16\n\x0emcEnterpriseId\x18\x02 \x01(\x05\x12\x17\n\x0fmaxLicenseCount\x18\x04 \x01(\x05\x12\x19\n\x11maxFilePlanTypeId\x18\x05 \x01(\x05\x12\x15\n\rmaxBasePlanId\x18\x06 \x01(\x05\x12(\n\x06\x61\x64\x64ons\x18\x07 \x03(\x0b\x32\x18.BI.SnapshotRecord.Addon\x1a*\n\x05\x41\x64\x64on\x12\x12\n\nmaxAddonId\x18\x01 \x01(\x05\x12\r\n\x05units\x18\x02 \x01(\x03\"0\n\x14SnapshotMcEnterprise\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\"\x16\n\x14MappingAddonsRequest\"\\\n\x15MappingAddonsResponse\x12\x1f\n\x06\x61\x64\x64ons\x18\x01 \x03(\x0b\x32\x0f.BI.MappingItem\x12\"\n\tfilePlans\x18\x02 \x03(\x0b\x32\x0f.BI.MappingItem\"\'\n\x0bMappingItem\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\"1\n\x1aGradientValidateKeyRequest\x12\x13\n\x0bgradientKey\x18\x01 \x01(\t\"?\n\x1bGradientValidateKeyResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"D\n\x13GradientSaveRequest\x12\x13\n\x0bgradientKey\x18\x01 \x01(\t\x12\x18\n\x10\x65nterpriseUserId\x18\x02 \x01(\x03\"g\n\x14GradientSaveResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12-\n\x06status\x18\x02 \x01(\x0e\x32\x1d.BI.GradientIntegrationStatus\x12\x0f\n\x07message\x18\x03 \x01(\t\"1\n\x15GradientRemoveRequest\x12\x18\n\x10\x65nterpriseUserId\x18\x01 \x01(\x03\":\n\x16GradientRemoveResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"/\n\x13GradientSyncRequest\x12\x18\n\x10\x65nterpriseUserId\x18\x01 \x01(\x03\"g\n\x14GradientSyncResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12-\n\x06status\x18\x02 \x01(\x0e\x32\x1d.BI.GradientIntegrationStatus\x12\x0f\n\x07message\x18\x03 \x01(\t\"N\n\'NetPromoterScoreSurveySubmissionRequest\x12\x14\n\x0csurvey_score\x18\x01 \x01(\x05\x12\r\n\x05notes\x18\x02 \x01(\t\"*\n(NetPromoterScoreSurveySubmissionResponse\"&\n$NetPromoterScorePopupScheduleRequest\";\n%NetPromoterScorePopupScheduleResponse\x12\x12\n\nshow_popup\x18\x01 \x01(\x08\"\'\n%NetPromoterScorePopupDismissalRequest\"(\n&NetPromoterScorePopupDismissalResponse\"-\n\x11KCMLicenseRequest\x12\x18\n\x10\x65nterpriseUserId\x18\x01 \x01(\x03\"%\n\x12KCMLicenseResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x84\x01\n\x0c\x45ventRequest\x12 \n\teventType\x18\x01 \x01(\x0e\x32\r.BI.EventType\x12\x12\n\neventValue\x18\x02 \x01(\t\x12\x11\n\teventTime\x18\x03 \x01(\x03\x12+\n\nattributes\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\"0\n\rEventsRequest\x12\x1f\n\x05\x65vent\x18\x01 \x03(\x0b\x32\x10.BI.EventRequest\"\x87\x01\n\x16\x43ustomerCaptureRequest\x12\x0f\n\x07pageUrl\x18\x01 \x01(\t\x12\x0c\n\x04tree\x18\x02 \x01(\t\x12\x0c\n\x04hash\x18\x03 \x01(\t\x12\r\n\x05image\x18\x04 \x01(\t\x12\x14\n\x0cpageLoadTime\x18\x05 \x01(\t\x12\r\n\x05keyId\x18\x06 \x01(\t\x12\x0c\n\x04test\x18\x07 \x01(\x08\"\x19\n\x17\x43ustomerCaptureResponse*M\n\x08\x43urrency\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03USD\x10\x01\x12\x07\n\x03GBP\x10\x02\x12\x07\n\x03JPY\x10\x03\x12\x07\n\x03\x45UR\x10\x04\x12\x07\n\x03\x41UD\x10\x05\x12\x07\n\x03\x43\x41\x44\x10\x06*S\n\x19GradientIntegrationStatus\x12\x10\n\x0cNOTCONNECTED\x10\x00\x12\x0b\n\x07PENDING\x10\x01\x12\r\n\tCONNECTED\x10\x02\x12\x08\n\x04NONE\x10\x03*\xb9\x01\n\tEventType\x12\x1f\n\x1bUNKNOWN_TRACKING_EVENT_TYPE\x10\x00\x12\x1c\n\x18TRACKING_POPUP_DISPLAYED\x10\x01\x12\x1b\n\x17TRACKING_POPUP_ACCEPTED\x10\x02\x12\x1c\n\x18TRACKING_POPUP_DISMISSED\x10\x03\x12\x17\n\x13TRACKING_POPUP_PAID\x10\x04\x12\x19\n\x15TRACKING_PUSH_CLICKED\x10\x05\x42\x1e\n\x18\x63om.keepersecurity.protoB\x02\x42Ib\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'BI_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030com.keepersecurity.protoB\002BI'
  _globals['_CURRENCY']._serialized_start=5253
  _globals['_CURRENCY']._serialized_end=5330
  _globals['_GRADIENTINTEGRATIONSTATUS']._serialized_start=5332
  _globals['_GRADIENTINTEGRATIONSTATUS']._serialized_end=5415
  _globals['_EVENTTYPE']._serialized_start=5418
  _globals['_EVENTTYPE']._serialized_end=5603
  _globals['_VALIDATESESSIONTOKENREQUEST']._serialized_start=46
  _globals['_VALIDATESESSIONTOKENREQUEST']._serialized_end=148
  _globals['_VALIDATESESSIONTOKENRESPONSE']._serialized_start=151
  _globals['_VALIDATESESSIONTOKENRESPONSE']._serialized_end=497
  _globals['_VALIDATESESSIONTOKENRESPONSE_STATUS']._serialized_start=406
  _globals['_VALIDATESESSIONTOKENRESPONSE_STATUS']._serialized_end=497
  _globals['_SUBSCRIPTIONSTATUSREQUEST']._serialized_start=499
  _globals['_SUBSCRIPTIONSTATUSREQUEST']._serialized_end=526
  _globals['_SUBSCRIPTIONSTATUSRESPONSE']._serialized_start=529
  _globals['_SUBSCRIPTIONSTATUSRESPONSE']._serialized_end=951
  _globals['_LICENSESTATS']._serialized_start=954
  _globals['_LICENSESTATS']._serialized_end=1169
  _globals['_LICENSESTATS_TYPE']._serialized_start=1041
  _globals['_LICENSESTATS_TYPE']._serialized_end=1169
  _globals['_AUTORENEWAL']._serialized_start=1171
  _globals['_AUTORENEWAL']._serialized_end=1235
  _globals['_PAYMENTMETHOD']._serialized_start=1238
  _globals['_PAYMENTMETHOD']._serialized_end=1754
  _globals['_PAYMENTMETHOD_CARD']._serialized_start=1532
  _globals['_PAYMENTMETHOD_CARD']._serialized_end=1568
  _globals['_PAYMENTMETHOD_SEPA']._serialized_start=1570
  _globals['_PAYMENTMETHOD_SEPA']._serialized_end=1608
  _globals['_PAYMENTMETHOD_PAYPAL']._serialized_start=1610
  _globals['_PAYMENTMETHOD_PAYPAL']._serialized_end=1618
  _globals['_PAYMENTMETHOD_VENDOR']._serialized_start=1620
  _globals['_PAYMENTMETHOD_VENDOR']._serialized_end=1642
  _globals['_PAYMENTMETHOD_PURCHASEORDER']._serialized_start=1644
  _globals['_PAYMENTMETHOD_PURCHASEORDER']._serialized_end=1673
  _globals['_PAYMENTMETHOD_TYPE']._serialized_start=1675
  _globals['_PAYMENTMETHOD_TYPE']._serialized_end=1754
  _globals['_SUBSCRIPTIONMSPPRICINGREQUEST']._serialized_start=1756
  _globals['_SUBSCRIPTIONMSPPRICINGREQUEST']._serialized_end=1787
  _globals['_SUBSCRIPTIONMSPPRICINGRESPONSE']._serialized_start=1789
  _globals['_SUBSCRIPTIONMSPPRICINGRESPONSE']._serialized_end=1881
  _globals['_SUBSCRIPTIONMCPRICINGREQUEST']._serialized_start=1883
  _globals['_SUBSCRIPTIONMCPRICINGREQUEST']._serialized_end=1913
  _globals['_SUBSCRIPTIONMCPRICINGRESPONSE']._serialized_start=1915
  _globals['_SUBSCRIPTIONMCPRICINGRESPONSE']._serialized_end=2039
  _globals['_BASEPLAN']._serialized_start=2041
  _globals['_BASEPLAN']._serialized_end=2087
  _globals['_ADDON']._serialized_start=2089
  _globals['_ADDON']._serialized_end=2156
  _globals['_FILEPLAN']._serialized_start=2158
  _globals['_FILEPLAN']._serialized_end=2204
  _globals['_COST']._serialized_start=2207
  _globals['_COST']._serialized_end=2398
  _globals['_COST_AMOUNTPER']._serialized_start=2302
  _globals['_COST_AMOUNTPER']._serialized_end=2398
  _globals['_INVOICESEARCHREQUEST']._serialized_start=2400
  _globals['_INVOICESEARCHREQUEST']._serialized_end=2492
  _globals['_INVOICESEARCHRESPONSE']._serialized_start=2494
  _globals['_INVOICESEARCHRESPONSE']._serialized_end=2548
  _globals['_INVOICE']._serialized_start=2551
  _globals['_INVOICE']._serialized_end=2869
  _globals['_INVOICE_COST']._serialized_start=2716
  _globals['_INVOICE_COST']._serialized_end=2770
  _globals['_INVOICE_TYPE']._serialized_start=2772
  _globals['_INVOICE_TYPE']._serialized_end=2869
  _globals['_VAULTINVOICESLISTREQUEST']._serialized_start=2871
  _globals['_VAULTINVOICESLISTREQUEST']._serialized_end=2897
  _globals['_VAULTINVOICESLISTRESPONSE']._serialized_start=2899
  _globals['_VAULTINVOICESLISTRESPONSE']._serialized_end=2962
  _globals['_VAULTINVOICE']._serialized_start=2965
  _globals['_VAULTINVOICE']._serialized_end=3108
  _globals['_INVOICEDOWNLOADREQUEST']._serialized_start=3110
  _globals['_INVOICEDOWNLOADREQUEST']._serialized_end=3157
  _globals['_INVOICEDOWNLOADRESPONSE']._serialized_start=3159
  _globals['_INVOICEDOWNLOADRESPONSE']._serialized_end=3216
  _globals['_VAULTINVOICEDOWNLOADLINKREQUEST']._serialized_start=3218
  _globals['_VAULTINVOICEDOWNLOADLINKREQUEST']._serialized_end=3274
  _globals['_VAULTINVOICEDOWNLOADLINKRESPONSE']._serialized_start=3276
  _globals['_VAULTINVOICEDOWNLOADLINKRESPONSE']._serialized_end=3342
  _globals['_REPORTINGDAILYSNAPSHOTREQUEST']._serialized_start=3344
  _globals['_REPORTINGDAILYSNAPSHOTREQUEST']._serialized_end=3404
  _globals['_REPORTINGDAILYSNAPSHOTRESPONSE']._serialized_start=3406
  _globals['_REPORTINGDAILYSNAPSHOTRESPONSE']._serialized_end=3524
  _globals['_SNAPSHOTRECORD']._serialized_start=3527
  _globals['_SNAPSHOTRECORD']._serialized_end=3742
  _globals['_SNAPSHOTRECORD_ADDON']._serialized_start=3700
  _globals['_SNAPSHOTRECORD_ADDON']._serialized_end=3742
  _globals['_SNAPSHOTMCENTERPRISE']._serialized_start=3744
  _globals['_SNAPSHOTMCENTERPRISE']._serialized_end=3792
  _globals['_MAPPINGADDONSREQUEST']._serialized_start=3794
  _globals['_MAPPINGADDONSREQUEST']._serialized_end=3816
  _globals['_MAPPINGADDONSRESPONSE']._serialized_start=3818
  _globals['_MAPPINGADDONSRESPONSE']._serialized_end=3910
  _globals['_MAPPINGITEM']._serialized_start=3912
  _globals['_MAPPINGITEM']._serialized_end=3951
  _globals['_GRADIENTVALIDATEKEYREQUEST']._serialized_start=3953
  _globals['_GRADIENTVALIDATEKEYREQUEST']._serialized_end=4002
  _globals['_GRADIENTVALIDATEKEYRESPONSE']._serialized_start=4004
  _globals['_GRADIENTVALIDATEKEYRESPONSE']._serialized_end=4067
  _globals['_GRADIENTSAVEREQUEST']._serialized_start=4069
  _globals['_GRADIENTSAVEREQUEST']._serialized_end=4137
  _globals['_GRADIENTSAVERESPONSE']._serialized_start=4139
  _globals['_GRADIENTSAVERESPONSE']._serialized_end=4242
  _globals['_GRADIENTREMOVEREQUEST']._serialized_start=4244
  _globals['_GRADIENTREMOVEREQUEST']._serialized_end=4293
  _globals['_GRADIENTREMOVERESPONSE']._serialized_start=4295
  _globals['_GRADIENTREMOVERESPONSE']._serialized_end=4353
  _globals['_GRADIENTSYNCREQUEST']._serialized_start=4355
  _globals['_GRADIENTSYNCREQUEST']._serialized_end=4402
  _globals['_GRADIENTSYNCRESPONSE']._serialized_start=4404
  _globals['_GRADIENTSYNCRESPONSE']._serialized_end=4507
  _globals['_NETPROMOTERSCORESURVEYSUBMISSIONREQUEST']._serialized_start=4509
  _globals['_NETPROMOTERSCORESURVEYSUBMISSIONREQUEST']._serialized_end=4587
  _globals['_NETPROMOTERSCORESURVEYSUBMISSIONRESPONSE']._serialized_start=4589
  _globals['_NETPROMOTERSCORESURVEYSUBMISSIONRESPONSE']._serialized_end=4631
  _globals['_NETPROMOTERSCOREPOPUPSCHEDULEREQUEST']._serialized_start=4633
  _globals['_NETPROMOTERSCOREPOPUPSCHEDULEREQUEST']._serialized_end=4671
  _globals['_NETPROMOTERSCOREPOPUPSCHEDULERESPONSE']._serialized_start=4673
  _globals['_NETPROMOTERSCOREPOPUPSCHEDULERESPONSE']._serialized_end=4732
  _globals['_NETPROMOTERSCOREPOPUPDISMISSALREQUEST']._serialized_start=4734
  _globals['_NETPROMOTERSCOREPOPUPDISMISSALREQUEST']._serialized_end=4773
  _globals['_NETPROMOTERSCOREPOPUPDISMISSALRESPONSE']._serialized_start=4775
  _globals['_NETPROMOTERSCOREPOPUPDISMISSALRESPONSE']._serialized_end=4815
  _globals['_KCMLICENSEREQUEST']._serialized_start=4817
  _globals['_KCMLICENSEREQUEST']._serialized_end=4862
  _globals['_KCMLICENSERESPONSE']._serialized_start=4864
  _globals['_KCMLICENSERESPONSE']._serialized_end=4901
  _globals['_EVENTREQUEST']._serialized_start=4904
  _globals['_EVENTREQUEST']._serialized_end=5036
  _globals['_EVENTSREQUEST']._serialized_start=5038
  _globals['_EVENTSREQUEST']._serialized_end=5086
  _globals['_CUSTOMERCAPTUREREQUEST']._serialized_start=5089
  _globals['_CUSTOMERCAPTUREREQUEST']._serialized_end=5224
  _globals['_CUSTOMERCAPTURERESPONSE']._serialized_start=5226
  _globals['_CUSTOMERCAPTURERESPONSE']._serialized_end=5251
# @@protoc_insertion_point(module_scope)
