# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: breachwatch.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x62reachwatch.proto\x12\x0b\x42reachWatch\"\xa1\x01\n\x18\x42reachWatchRecordRequest\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x15\n\rencryptedData\x18\x02 \x01(\x0c\x12=\n\x13\x62reachWatchInfoType\x18\x03 \x01(\x0e\x32 .BreachWatch.BreachWatchInfoType\x12\x1c\n\x14updateUserWhoScanned\x18\x04 \x01(\x08\"z\n\x18\x42reachWatchUpdateRequest\x12G\n\x18\x62reachWatchRecordRequest\x18\x01 \x03(\x0b\x32%.BreachWatch.BreachWatchRecordRequest\x12\x15\n\rencryptedData\x18\x02 \x01(\x0c\"L\n\x17\x42reachWatchRecordStatus\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x0e\n\x06reason\x18\x03 \x01(\t\"b\n\x19\x42reachWatchUpdateResponse\x12\x45\n\x17\x62reachWatchRecordStatus\x18\x01 \x03(\x0b\x32$.BreachWatch.BreachWatchRecordStatus\"3\n\x17\x42reachWatchTokenRequest\x12\x18\n\x10\x62reachWatchToken\x18\x01 \x01(\x0c\"M\n\x18\x42reachWatchTokenResponse\x12\x18\n\x10\x62reachWatchToken\x18\x01 \x01(\x0c\x12\x17\n\x0f\x63lientEncrypted\x18\x02 \x01(\x08\"Y\n\x17\x41nonymizedTokenResponse\x12\x13\n\x0b\x64omainToken\x18\x01 \x01(\x0c\x12\x12\n\nemailToken\x18\x02 \x01(\x0c\x12\x15\n\rpasswordToken\x18\x03 \x01(\x0c\"(\n\tHashCheck\x12\r\n\x05hash1\x18\x01 \x01(\x0c\x12\x0c\n\x04\x65uid\x18\x02 \x01(\x0c\"s\n\x18\x42reachWatchStatusRequest\x12\x17\n\x0f\x61nonymizedToken\x18\x01 \x01(\x0c\x12)\n\thashCheck\x18\x02 \x03(\x0b\x32\x16.BreachWatch.HashCheck\x12\x13\n\x0bremovedEuid\x18\x03 \x03(\x0c\"A\n\nHashStatus\x12\r\n\x05hash1\x18\x01 \x01(\x0c\x12\x0c\n\x04\x65uid\x18\x02 \x01(\x0c\x12\x16\n\x0e\x62reachDetected\x18\x03 \x01(\x08\"H\n\x19\x42reachWatchStatusResponse\x12+\n\nhashStatus\x18\x02 \x03(\x0b\x32\x17.BreachWatch.HashStatus\"Z\n\x1b\x45nterprisePublicKeyResponse\x12\x1b\n\x13\x65nterprisePublicKey\x18\x01 \x01(\x0c\x12\x1e\n\x16\x65nterpriseECCPublicKey\x18\x02 \x01(\x0c\"&\n\x0f\x46reeScanRequest\x12\x13\n\x0bhashedEmail\x18\x01 \x01(\x0c\"C\n\x10\x46reeScanResponse\x12\x15\n\remailBreaches\x18\x01 \x01(\x05\x12\x18\n\x10passwordBreaches\x18\x02 \x01(\x05\" \n\x0fPaidUserRequest\x12\r\n\x05\x65mail\x18\x01 \x01(\t\"$\n\x10PaidUserResponse\x12\x10\n\x08paidUser\x18\x01 \x01(\x08\"$\n\x13\x44\x65tailedScanRequest\x12\r\n\x05\x65mail\x18\x01 \x01(\t\"\'\n\x16UseOneTimeTokenRequest\x12\r\n\x05token\x18\x01 \x01(\x0c\"g\n\x0b\x42reachEvent\x12\x0c\n\x04site\x18\x01 \x01(\t\x12\r\n\x05\x65mail\x18\x02 \x01(\t\x12\x18\n\x10passwordInBreach\x18\x03 \x01(\x08\x12\x0c\n\x04\x64\x61te\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\"\x89\x01\n\x17UseOneTimeTokenResponse\x12\x15\n\remailBreaches\x18\x01 \x01(\x05\x12\x18\n\x10passwordBreaches\x18\x02 \x01(\x05\x12.\n\x0c\x62reachEvents\x18\x03 \x03(\x0b\x32\x18.BreachWatch.BreachEvent\x12\r\n\x05\x65mail\x18\x04 \x01(\t\"-\n\x0fOneTimeUseToken\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x0b\n\x03pad\x18\x02 \x01(\t\"1\n\x17\x46reePasswordScanRequest\x12\x16\n\x0ehashedPassword\x18\x01 \x01(\x0c\"4\n\x18\x46reePasswordScanResponse\x12\x18\n\x10passwordBreaches\x18\x01 \x01(\x03*9\n\x13\x42reachWatchInfoType\x12\n\n\x06RECORD\x10\x00\x12\x16\n\x12\x41LTERNATE_PASSWORD\x10\x01\x42\'\n\x18\x63om.keepersecurity.protoB\x0b\x42reachWatchb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'breachwatch_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030com.keepersecurity.protoB\013BreachWatch'
  _globals['_BREACHWATCHINFOTYPE']._serialized_start=1772
  _globals['_BREACHWATCHINFOTYPE']._serialized_end=1829
  _globals['_BREACHWATCHRECORDREQUEST']._serialized_start=35
  _globals['_BREACHWATCHRECORDREQUEST']._serialized_end=196
  _globals['_BREACHWATCHUPDATEREQUEST']._serialized_start=198
  _globals['_BREACHWATCHUPDATEREQUEST']._serialized_end=320
  _globals['_BREACHWATCHRECORDSTATUS']._serialized_start=322
  _globals['_BREACHWATCHRECORDSTATUS']._serialized_end=398
  _globals['_BREACHWATCHUPDATERESPONSE']._serialized_start=400
  _globals['_BREACHWATCHUPDATERESPONSE']._serialized_end=498
  _globals['_BREACHWATCHTOKENREQUEST']._serialized_start=500
  _globals['_BREACHWATCHTOKENREQUEST']._serialized_end=551
  _globals['_BREACHWATCHTOKENRESPONSE']._serialized_start=553
  _globals['_BREACHWATCHTOKENRESPONSE']._serialized_end=630
  _globals['_ANONYMIZEDTOKENRESPONSE']._serialized_start=632
  _globals['_ANONYMIZEDTOKENRESPONSE']._serialized_end=721
  _globals['_HASHCHECK']._serialized_start=723
  _globals['_HASHCHECK']._serialized_end=763
  _globals['_BREACHWATCHSTATUSREQUEST']._serialized_start=765
  _globals['_BREACHWATCHSTATUSREQUEST']._serialized_end=880
  _globals['_HASHSTATUS']._serialized_start=882
  _globals['_HASHSTATUS']._serialized_end=947
  _globals['_BREACHWATCHSTATUSRESPONSE']._serialized_start=949
  _globals['_BREACHWATCHSTATUSRESPONSE']._serialized_end=1021
  _globals['_ENTERPRISEPUBLICKEYRESPONSE']._serialized_start=1023
  _globals['_ENTERPRISEPUBLICKEYRESPONSE']._serialized_end=1113
  _globals['_FREESCANREQUEST']._serialized_start=1115
  _globals['_FREESCANREQUEST']._serialized_end=1153
  _globals['_FREESCANRESPONSE']._serialized_start=1155
  _globals['_FREESCANRESPONSE']._serialized_end=1222
  _globals['_PAIDUSERREQUEST']._serialized_start=1224
  _globals['_PAIDUSERREQUEST']._serialized_end=1256
  _globals['_PAIDUSERRESPONSE']._serialized_start=1258
  _globals['_PAIDUSERRESPONSE']._serialized_end=1294
  _globals['_DETAILEDSCANREQUEST']._serialized_start=1296
  _globals['_DETAILEDSCANREQUEST']._serialized_end=1332
  _globals['_USEONETIMETOKENREQUEST']._serialized_start=1334
  _globals['_USEONETIMETOKENREQUEST']._serialized_end=1373
  _globals['_BREACHEVENT']._serialized_start=1375
  _globals['_BREACHEVENT']._serialized_end=1478
  _globals['_USEONETIMETOKENRESPONSE']._serialized_start=1481
  _globals['_USEONETIMETOKENRESPONSE']._serialized_end=1618
  _globals['_ONETIMEUSETOKEN']._serialized_start=1620
  _globals['_ONETIMEUSETOKEN']._serialized_end=1665
  _globals['_FREEPASSWORDSCANREQUEST']._serialized_start=1667
  _globals['_FREEPASSWORDSCANREQUEST']._serialized_end=1716
  _globals['_FREEPASSWORDSCANRESPONSE']._serialized_start=1718
  _globals['_FREEPASSWORDSCANRESPONSE']._serialized_end=1770
# @@protoc_insertion_point(module_scope)
