# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: GraphSync.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fGraphSync.proto\x12\tGraphSync\"M\n\x0cGraphSyncRef\x12 \n\x04type\x18\x01 \x01(\x0e\x32\x12.GraphSync.RefType\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x0c\n\x04name\x18\x03 \x01(\t\"p\n\x0eGraphSyncActor\x12+\n\x04type\x18\x01 \x01(\x0e\x32\x1d.GraphSync.GraphSyncActorType\x12\n\n\x02id\x18\x02 \x01(\x0c\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x17\n\x0f\x65\x66\x66\x65\x63tiveUserId\x18\x04 \x01(\x0c\"\xac\x01\n\rGraphSyncData\x12*\n\x04type\x18\x01 \x01(\x0e\x32\x1c.GraphSync.GraphSyncDataType\x12$\n\x03ref\x18\x02 \x01(\x0b\x32\x17.GraphSync.GraphSyncRef\x12*\n\tparentRef\x18\x03 \x01(\x0b\x32\x17.GraphSync.GraphSyncRef\x12\x0f\n\x07\x63ontent\x18\x04 \x01(\x0c\x12\x0c\n\x04path\x18\x05 \x01(\t\"x\n\x11GraphSyncDataPlus\x12&\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x18.GraphSync.GraphSyncData\x12\x11\n\ttimestamp\x18\x02 \x01(\x03\x12(\n\x05\x61\x63tor\x18\x03 \x01(\x0b\x32\x19.GraphSync.GraphSyncActor\"W\n\x0eGraphSyncQuery\x12\x10\n\x08streamId\x18\x01 \x01(\x0c\x12\x0e\n\x06origin\x18\x02 \x01(\x0c\x12\x11\n\tsyncPoint\x18\x03 \x01(\x03\x12\x10\n\x08maxCount\x18\x04 \x01(\x05\"s\n\x0fGraphSyncResult\x12\x10\n\x08streamId\x18\x02 \x01(\x0c\x12\x11\n\tsyncPoint\x18\x03 \x01(\x03\x12*\n\x04\x64\x61ta\x18\x04 \x03(\x0b\x32\x1c.GraphSync.GraphSyncDataPlus\x12\x0f\n\x07hasMore\x18\x05 \x01(\x08\"A\n\x13GraphSyncMultiQuery\x12*\n\x07queries\x18\x01 \x03(\x0b\x32\x19.GraphSync.GraphSyncQuery\"C\n\x14GraphSyncMultiResult\x12+\n\x07results\x18\x01 \x03(\x0b\x32\x1a.GraphSync.GraphSyncResult\"j\n\x17GraphSyncAddDataRequest\x12\'\n\x06origin\x18\x01 \x01(\x0b\x32\x17.GraphSync.GraphSyncRef\x12&\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32\x18.GraphSync.GraphSyncData\"\'\n\x13GraphSyncLeafsQuery\x12\x10\n\x08vertices\x18\x02 \x03(\x0c\"<\n\x13GraphSyncRefsResult\x12%\n\x04refs\x18\x01 \x03(\x0b\x32\x17.GraphSync.GraphSyncRef*\xe5\x02\n\x07RefType\x12\x0f\n\x0bRFT_GENERAL\x10\x00\x12\x0c\n\x08RFT_USER\x10\x01\x12\x0e\n\nRFT_DEVICE\x10\x02\x12\x0b\n\x07RFT_REC\x10\x03\x12\x0e\n\nRFT_FOLDER\x10\x04\x12\x0c\n\x08RFT_TEAM\x10\x05\x12\x12\n\x0eRFT_ENTERPRISE\x10\x06\x12\x15\n\x11RFT_PAM_DIRECTORY\x10\x07\x12\x13\n\x0fRFT_PAM_MACHINE\x10\x08\x12\x14\n\x10RFT_PAM_DATABASE\x10\t\x12\x10\n\x0cRFT_PAM_USER\x10\n\x12\x13\n\x0fRFT_PAM_NETWORK\x10\x0b\x12\x13\n\x0fRFT_PAM_BROWSER\x10\x0c\x12\x12\n\x0eRFT_CONNECTION\x10\r\x12\x10\n\x0cRFT_WORKFLOW\x10\x0e\x12\x14\n\x10RFT_NOTIFICATION\x10\x0f\x12\x11\n\rRFT_USER_INFO\x10\x10\x12\x11\n\rRFT_TEAM_INFO\x10\x11\x12\x0c\n\x08RFT_ROLE\x10\x12*[\n\x11GraphSyncDataType\x12\x0c\n\x08GSE_DATA\x10\x00\x12\x0b\n\x07GSE_KEY\x10\x01\x12\x0c\n\x08GSE_LINK\x10\x02\x12\x0b\n\x07GSE_ACL\x10\x03\x12\x10\n\x0cGSE_DELETION\x10\x04*H\n\x12GraphSyncActorType\x12\x0c\n\x08GSA_USER\x10\x00\x12\x0f\n\x0bGSA_SERVICE\x10\x01\x12\x13\n\x0fGSA_PAM_GATEWAY\x10\x02\x42%\n\x18\x63om.keepersecurity.protoB\tGraphSyncb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'GraphSync_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030com.keepersecurity.protoB\tGraphSync'
  _globals['_REFTYPE']._serialized_start=1074
  _globals['_REFTYPE']._serialized_end=1431
  _globals['_GRAPHSYNCDATATYPE']._serialized_start=1433
  _globals['_GRAPHSYNCDATATYPE']._serialized_end=1524
  _globals['_GRAPHSYNCACTORTYPE']._serialized_start=1526
  _globals['_GRAPHSYNCACTORTYPE']._serialized_end=1598
  _globals['_GRAPHSYNCREF']._serialized_start=30
  _globals['_GRAPHSYNCREF']._serialized_end=107
  _globals['_GRAPHSYNCACTOR']._serialized_start=109
  _globals['_GRAPHSYNCACTOR']._serialized_end=221
  _globals['_GRAPHSYNCDATA']._serialized_start=224
  _globals['_GRAPHSYNCDATA']._serialized_end=396
  _globals['_GRAPHSYNCDATAPLUS']._serialized_start=398
  _globals['_GRAPHSYNCDATAPLUS']._serialized_end=518
  _globals['_GRAPHSYNCQUERY']._serialized_start=520
  _globals['_GRAPHSYNCQUERY']._serialized_end=607
  _globals['_GRAPHSYNCRESULT']._serialized_start=609
  _globals['_GRAPHSYNCRESULT']._serialized_end=724
  _globals['_GRAPHSYNCMULTIQUERY']._serialized_start=726
  _globals['_GRAPHSYNCMULTIQUERY']._serialized_end=791
  _globals['_GRAPHSYNCMULTIRESULT']._serialized_start=793
  _globals['_GRAPHSYNCMULTIRESULT']._serialized_end=860
  _globals['_GRAPHSYNCADDDATAREQUEST']._serialized_start=862
  _globals['_GRAPHSYNCADDDATAREQUEST']._serialized_end=968
  _globals['_GRAPHSYNCLEAFSQUERY']._serialized_start=970
  _globals['_GRAPHSYNCLEAFSQUERY']._serialized_end=1009
  _globals['_GRAPHSYNCREFSRESULT']._serialized_start=1011
  _globals['_GRAPHSYNCREFSRESULT']._serialized_end=1071
# @@protoc_insertion_point(module_scope)
