# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rmd.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\trmd.proto\x12\x03RMD\"D\n\x0e\x45nterpriseStat\x12\x19\n\x11usersLoggedRecent\x18\x01 \x01(\x05\x12\x17\n\x0fusersHasRecords\x18\x02 \x01(\x05\"Z\n\x14\x45nterpriseStatDetail\x12\x18\n\x10\x65nterpriseUserId\x18\x01 \x01(\x03\x12\x14\n\x0clastLoggedIn\x18\x02 \x01(\x03\x12\x12\n\nhasRecords\x18\x03 \x01(\x08\"P\n\x1f\x45nterpriseStatContinuationToken\x12\x13\n\x0blastUpdated\x18\x01 \x01(\x03\x12\x18\n\x10\x65nterpriseUserId\x18\x02 \x01(\x03\"t\n\x1c\x45nterpriseStatDetailsRequest\x12\x13\n\x0blastUpdated\x18\x01 \x01(\x03\x12?\n\x11\x63ontinuationToken\x18\x02 \x01(\x0b\x32$.RMD.EnterpriseStatContinuationToken\"\xc0\x01\n\x1d\x45nterpriseStatDetailsResponse\x12\x38\n\x15\x65nterpriseStatDetails\x18\x01 \x03(\x0b\x32\x19.RMD.EnterpriseStatDetail\x12\x13\n\x0blastUpdated\x18\x02 \x01(\x03\x12?\n\x11\x63ontinuationToken\x18\x03 \x01(\x0b\x32$.RMD.EnterpriseStatContinuationToken\x12\x0f\n\x07hasMore\x18\x04 \x01(\x08\"\x93\x01\n\x15SecurityAlertsSummary\x12\x18\n\x10\x61uditEventTypeId\x18\x01 \x01(\x05\x12\x14\n\x0c\x63urrentCount\x18\x02 \x01(\x05\x12\x18\n\x10\x63urrentUserCount\x18\x03 \x01(\x05\x12\x15\n\rpreviousCount\x18\x04 \x01(\x05\x12\x19\n\x11previousUserCount\x18\x05 \x01(\x05\"Z\n\x1dSecurityAlertsSummaryResponse\x12\x39\n\x15securityAlertsSummary\x18\x01 \x03(\x0b\x32\x1a.RMD.SecurityAlertsSummary\"R\n\x1bSecurityAlertsDetailRequest\x12\x18\n\x10\x61uditEventTypeId\x18\x01 \x01(\x05\x12\x19\n\x11\x63ontinuationToken\x18\x02 \x01(\x03\"u\n\x14SecurityAlertsDetail\x12\x18\n\x10\x65nterpriseUserId\x18\x01 \x01(\x03\x12\x14\n\x0c\x63urrentCount\x18\x02 \x01(\x05\x12\x15\n\rpreviousCount\x18\x03 \x01(\x05\x12\x16\n\x0elastOccurrence\x18\x04 \x01(\x03\"\x83\x01\n\x1cSecurityAlertsDetailResponse\x12\x37\n\x14securityAlertDetails\x18\x01 \x03(\x0b\x32\x19.RMD.SecurityAlertsDetail\x12\x0f\n\x07hasMore\x18\x02 \x01(\x08\x12\x19\n\x11\x63ontinuationToken\x18\x03 \x01(\x03\"\xb9\x01\n\x1b\x45nterpriseSecurityBenchmark\x12\x31\n\x11securityBenchmark\x18\x01 \x01(\x0e\x32\x16.RMD.SecurityBenchmark\x12=\n\x17securityBenchmarkStatus\x18\x02 \x01(\x0e\x32\x1c.RMD.SecurityBenchmarkStatus\x12\x13\n\x0blastUpdated\x18\x03 \x01(\x03\x12\x13\n\x0b\x61utoResolve\x18\x04 \x01(\x08\"f\n\x1cSetSecurityBenchmarksRequest\x12\x46\n\x1c\x65nterpriseSecurityBenchmarks\x18\x01 \x03(\x0b\x32 .RMD.EnterpriseSecurityBenchmark\"g\n\x1dGetSecurityBenchmarksResponse\x12\x46\n\x1c\x65nterpriseSecurityBenchmarks\x18\x01 \x03(\x0b\x32 .RMD.EnterpriseSecurityBenchmark*\xe7\x04\n\x11SecurityBenchmark\x12\x0e\n\nSB_INVALID\x10\x00\x12(\n$SB_DEPLOY_ACROSS_ENTIRE_ORGANIZATION\x10\x01\x12\x33\n/SB_PREVENT_INSTALLATION_OF_UNTRUSTED_EXTENSIONS\x10\x02\x12%\n!SB_ENABLE_ACCOUNT_TRANSFER_POLICY\x10\x03\x12%\n!SB_REDUCE_ADMINISTRATOR_PRIVILEGE\x10\x04\x12.\n*SB_ENSURE_OUTSIDE_SSO_ADMINISTRATOR_EXISTS\x10\x05\x12\x1d\n\x19SB_LOCK_DOWN_SSO_PROVIDER\x10\x06\x12\x30\n,SB_CREATE_AT_LEAST_TWO_KEEPER_ADMINISTRATORS\x10\x07\x12\x33\n/SB_ENSURE_TWO_FACTOR_AUTHENTICATION_ADMIN_USERS\x10\x08\x12%\n!SB_ENFORCE_STRONG_MASTER_PASSWORD\x10\t\x12\x35\n1SB_ENSURE_TWO_FACTOR_AUTHENTICATION_FOR_END_USERS\x10\n\x12 \n\x1cSB_CONFIGURE_IP_ALLOWLISTING\x10\x0b\x12\x14\n\x10SB_CREATE_ALERTS\x10\x0c\x12(\n$SB_DISABLE_BROWSER_PASSWORD_MANAGERS\x10\r\x12\x1f\n\x1bSB_DISABLE_ACCOUNT_RECOVERY\x10\x0e*Q\n\x17SecurityBenchmarkStatus\x12\x0b\n\x07INVALID\x10\x00\x12\x0c\n\x08RESOLVED\x10\x01\x12\x0b\n\x07IGNORED\x10\x02\x12\x0e\n\nUNRESOLVED\x10\x03\x42\x1f\n\x18\x63om.keepersecurity.protoB\x03RMDb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'rmd_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030com.keepersecurity.protoB\003RMD'
  _globals['_SECURITYBENCHMARK']._serialized_start=1552
  _globals['_SECURITYBENCHMARK']._serialized_end=2167
  _globals['_SECURITYBENCHMARKSTATUS']._serialized_start=2169
  _globals['_SECURITYBENCHMARKSTATUS']._serialized_end=2250
  _globals['_ENTERPRISESTAT']._serialized_start=18
  _globals['_ENTERPRISESTAT']._serialized_end=86
  _globals['_ENTERPRISESTATDETAIL']._serialized_start=88
  _globals['_ENTERPRISESTATDETAIL']._serialized_end=178
  _globals['_ENTERPRISESTATCONTINUATIONTOKEN']._serialized_start=180
  _globals['_ENTERPRISESTATCONTINUATIONTOKEN']._serialized_end=260
  _globals['_ENTERPRISESTATDETAILSREQUEST']._serialized_start=262
  _globals['_ENTERPRISESTATDETAILSREQUEST']._serialized_end=378
  _globals['_ENTERPRISESTATDETAILSRESPONSE']._serialized_start=381
  _globals['_ENTERPRISESTATDETAILSRESPONSE']._serialized_end=573
  _globals['_SECURITYALERTSSUMMARY']._serialized_start=576
  _globals['_SECURITYALERTSSUMMARY']._serialized_end=723
  _globals['_SECURITYALERTSSUMMARYRESPONSE']._serialized_start=725
  _globals['_SECURITYALERTSSUMMARYRESPONSE']._serialized_end=815
  _globals['_SECURITYALERTSDETAILREQUEST']._serialized_start=817
  _globals['_SECURITYALERTSDETAILREQUEST']._serialized_end=899
  _globals['_SECURITYALERTSDETAIL']._serialized_start=901
  _globals['_SECURITYALERTSDETAIL']._serialized_end=1018
  _globals['_SECURITYALERTSDETAILRESPONSE']._serialized_start=1021
  _globals['_SECURITYALERTSDETAILRESPONSE']._serialized_end=1152
  _globals['_ENTERPRISESECURITYBENCHMARK']._serialized_start=1155
  _globals['_ENTERPRISESECURITYBENCHMARK']._serialized_end=1340
  _globals['_SETSECURITYBENCHMARKSREQUEST']._serialized_start=1342
  _globals['_SETSECURITYBENCHMARKSREQUEST']._serialized_end=1444
  _globals['_GETSECURITYBENCHMARKSRESPONSE']._serialized_start=1446
  _globals['_GETSECURITYBENCHMARKSRESPONSE']._serialized_end=1549
# @@protoc_insertion_point(module_scope)
